@@ -1,10 +1,10 @@
 package Net::DNS::RR::AAAA;
 
 #
-# $Id: AAAA.pm 1348 2015-05-27 09:55:13Z willem $
+# $Id: AAAA.pm 1382 2015-08-26 20:39:33Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1348 $)[1];
+$VERSION = (qw$LastChangedRevision: 1382 $)[1];
 
 
 use strict;
@@ -20,7 +20,7 @@ Net::DNS::RR::AAAA - DNS AAAA resource record
 use integer;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -28,23 +28,23 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
-	return '' unless $self->{address} && length $self->{address};
+	return '' unless $self->{address};
 	pack 'a16', $self->{address};
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	return '' unless $self->{address} && length $self->{address};
+	return '' unless $self->{address};
 	return $self->address_short;
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->address(shift);
@@ -70,16 +70,15 @@ sub address_short {
 
 sub address {
 	my $self = shift;
+	my $addr = shift;
 
-	return $self->address_long unless scalar @_;
+	return $self->address_long unless defined $addr;
 
-	my $argument = shift || '';
-	my @parse = split /:/, "0$argument";
-	$self = {} unless ref($self);
+	my @parse = split /:/, "0$addr";
 
 	if ( (@parse)[$#parse] =~ /\./ ) {			# embedded IPv4
 		my @ip4 = split /\./, pop(@parse);
-		my $rhs = pop(@ip4) || 0;
+		my $rhs = pop(@ip4);
 		my @ip6 = map { /./ ? hex($_) : (0) x ( 7 - @parse ) } @parse;
 		return $self->{address} = pack 'n6 C4', @ip6, @ip4, (0) x ( 3 - @ip4 ), $rhs;
 	}
