@@ -1,10 +1,10 @@
 package Net::DNS::RR::APL;
 
 #
-# $Id: APL.pm 1360 2015-06-15 09:58:53Z willem $
+# $Id: APL.pm 1382 2015-08-26 20:39:33Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1360 $)[1];
+$VERSION = (qw$LastChangedRevision: 1382 $)[1];
 
 
 use strict;
@@ -22,7 +22,7 @@ use integer;
 use Carp;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -42,7 +42,7 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	my $rdata = '';
@@ -56,7 +56,7 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
 	return '' unless $self->{aplist};
@@ -64,7 +64,7 @@ sub format_rdata {			## format rdata portion of RR string.
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->aplist(@_);
@@ -105,9 +105,9 @@ package Net::DNS::RR::APL::Item;
 
 sub negate {
 	my $bit = 0x80;
-	for ( shift->{negate} ||= 0 ) {
-		return $_ & $bit unless scalar @_;
-		my $set = $_ | $bit;
+	for ( shift->{negate} ) {
+		my $set = $bit | ( $_ ||= 0 );
+		return $bit & $_ unless scalar @_;
 		$_ = (shift) ? $set : ( $set ^ $bit );
 		return $_ & $bit;
 	}
@@ -118,7 +118,7 @@ sub family {
 	my $self = shift;
 
 	$self->{family} = 0 + shift if scalar @_;
-	return $self->{family} || 0;
+	$self->{family} || 0;
 }
 
 
@@ -126,7 +126,7 @@ sub prefix {
 	my $self = shift;
 
 	$self->{prefix} = 0 + shift if scalar @_;
-	return $self->{prefix} || 0;
+	$self->{prefix} || 0;
 }
 
 
