@@ -1,10 +1,10 @@
 package Net::DNS::RR::HIP;
 
 #
-# $Id: HIP.pm 1339 2015-04-16 13:01:17Z willem $
+# $Id: HIP.pm 1382 2015-08-26 20:39:33Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1339 $)[1];
+$VERSION = (qw$LastChangedRevision: 1382 $)[1];
 
 
 use strict;
@@ -24,7 +24,7 @@ use Net::DNS::DomainName;
 use MIME::Base64;
 
 
-sub decode_rdata {			## decode rdata from wire-format octet string
+sub _decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
 	my ( $data, $offset ) = @_;
 
@@ -43,7 +43,7 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 }
 
 
-sub encode_rdata {			## encode rdata as wire-format octet string
+sub _encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
 
 	return '' unless $self->{hitbin};
@@ -54,7 +54,7 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 }
 
 
-sub format_rdata {			## format rdata portion of RR string.
+sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
 	return '' unless $self->{hitbin};
@@ -64,7 +64,7 @@ sub format_rdata {			## format rdata portion of RR string.
 }
 
 
-sub parse_rdata {			## populate RR from rdata in argument list
+sub _parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
 	$self->$_(shift) for qw(pkalgorithm hit key);
@@ -76,7 +76,7 @@ sub pkalgorithm {
 	my $self = shift;
 
 	$self->{pkalgorithm} = 0 + shift if scalar @_;
-	return $self->{pkalgorithm} || 0;
+	$self->{pkalgorithm} || 0;
 }
 
 
@@ -100,7 +100,7 @@ sub key {
 	my $self = shift;
 
 	$self->keybin( MIME::Base64::decode( join "", @_ ) ) if scalar @_;
-	return MIME::Base64::encode( $self->keybin(), "" ) if defined wantarray;
+	MIME::Base64::encode( $self->keybin(), "" ) if defined wantarray;
 }
 
 
@@ -112,6 +112,8 @@ sub keybin {
 }
 
 
+sub pubkey { &key; }
+
 sub servers {
 	my $self = shift;
 
@@ -120,10 +122,8 @@ sub servers {
 	return map $_->name, @$servers if defined wantarray;
 }
 
-sub pubkey { &key; }			## historical
-
 sub rendezvousservers {			## historical
-	my @servers = &servers;
+	my @servers = &servers;					# uncoverable pod
 	\@servers;
 }
 
@@ -174,6 +174,9 @@ The hexadecimal representation of the host identity tag.
 
 The binary representation of the host identity tag.
 
+=head2 pubkey
+
+
 =head2 key
 
     $key = $rr->key;
