@@ -1,15 +1,15 @@
 package Net::DNS::Resolver::Recurse;
 
 #
-# $Id: Recurse.pm 1259 2014-09-08 10:33:49Z willem $
+# $Id: Recurse.pm 1382 2015-08-26 20:39:33Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1259 $)[1];
+$VERSION = (qw$LastChangedRevision: 1382 $)[1];
 
 
 =head1 NAME
 
-Net::DNS::Resolver::Recurse - Perform recursive DNS lookups
+Net::DNS::Resolver::Recurse - DNS recursive resolver
 
 
 =head1 SYNOPSIS
@@ -91,15 +91,14 @@ sub send {
 	my $original = pop(@_);					# sneaky extra argument needed
 	$original = $question unless ref($original);		# to preserve original request
 
-	my ( $head, @tail ) = $question->{owner}->label;
-	unless ($head) {
+	my ( $head, @tail ) = $question->{qname}->label;
+	unless ( defined $head ) {
 		return $root if $root;				# root servers cached indefinitely
 
 		my $defres = new Net::DNS::Resolver();
-		$defres->nameservers( $res->hints ) || $defres->nameservers( $res->_hints );
+		$defres->nameservers( $res->hints );
 
 		my $packet = $defres->send( '.', 'NS' );	# specified hint server
-		$res->{callback}->($packet) if $res->{callback};
 		my @auth = grep $_->type eq 'NS', $packet->answer, $packet->authority;
 		my %auth = map { lc $_->nsdname => 1 } @auth;
 		my @glue = grep $auth{lc $_->name}, $packet->additional;
@@ -107,47 +106,45 @@ sub send {
 		foreach ( grep $_->type eq 'A',	   @glue ) { push @{$glue{lc $_->name}}, $_->address }
 		foreach ( grep $_->type eq 'AAAA', @glue ) { push @{$glue{lc $_->name}}, $_->address }
 		my @ip = map @$_, values %glue;
-		return $root = $packet if @ip && $packet->header->aa;
 
+		$defres->nameservers( $res->_hints );
 		$defres->nameservers(@ip);
 		$defres->recurse(0);
-		foreach my $ns ( map $_->nsdname, @auth ) {
-			$defres->nameservers($ns) unless @ip;
-			$packet = $defres->send( '.', 'NS' );	# authoritative root server
-			$res->{callback}->($packet) if $res->{callback};
-			my @auth = grep $_->type eq 'NS', $packet->answer, $packet->authority;
-			my %auth = map { lc $_->nsdname => 1 } @auth;
-			my @glue = grep $auth{lc $_->name}, $packet->additional;
-			my @ip = grep $_->type eq 'A', @glue;
-			push @ip, grep $_->type eq 'AAAA', @glue;
-			return $root = $packet if @ip && @auth;
-		}
-		return $packet;
+		$packet = $defres->send( '.', 'NS' );		# authoritative server
+		$self->_callback($packet);
+		return $root = $packet;
 	}
 
-	my $domain = $question->qtype ne 'NULL' ? $original->qname : join '.', @tail;
+	my $domain = lc join( '.', @tail ) || '.';
 	my $nslist = $res->{cache}->{$domain} ||= [];
 	if ( scalar @$nslist ) {
-		print ";; using cached nameservers for $domain.\n" if $res->{debug};
+		$self->_diag(";; using cached nameservers for $domain\n");
 	} else {
-		my $packet = $res->send( $domain, 'NULL', 'ANY', $original ) || return;
-		return $packet unless $packet->header->rcode eq 'NOERROR';
+		$domain = lc $question->qname if $question->qtype ne 'NULL';
+		my $packet = $res->send( $domain, 'NULL', 'IN', $original );
+
+		# uncoverable branch true	# depends on external data
+		return unless $packet;
 
 		my @answer = $packet->answer;			# return authoritative answer
 		return $packet if $packet->header->aa && grep $_->name eq $original->qname, @answer;
 
 		my @auth = grep $_->type eq 'NS', $packet->answer, $packet->authority;
-		print ";; cache nameservers for $domain.\n" if $res->{debug} && scalar(@auth);
-		my %auth = map { lc $_->nsdname => 1 } @auth;
+		my %auth = map { lc $_->nsdname => lc $_->name } @auth;
 		my @glue = grep $auth{lc $_->name}, $packet->additional;
-
 		my %glue;
 		foreach ( grep $_->type eq 'A',	   @glue ) { push @{$glue{lc $_->name}}, $_->address }
 		foreach ( grep $_->type eq 'AAAA', @glue ) { push @{$glue{lc $_->name}}, $_->address }
-		@$nslist = values %glue;
 
-		my @noglue = grep !$glue{$_}, keys %auth;
-		push @$nslist, @noglue;
+		my %zone = reverse %auth;
+		foreach my $zone ( keys %zone ) {
+			$self->_diag(";; cache nameservers for $zone\n");
+			my @nsname = grep $auth{$_} eq $zone, keys %auth;
+			my @list = map $glue{$_} ? $glue{$_} : $_, @nsname;
+			@{$res->{cache}->{$zone}} = @list;
+			return $packet if length($zone) > length($domain);
+			push @$nslist, @list;
+		}
 	}
 
 	my $query = new Net::DNS::Packet();
@@ -158,26 +155,45 @@ sub send {
 	splice @a, 0, 0, splice( @a, int( rand scalar @a ) );	# cut deck
 
 	foreach (@a) {
-		$res->nameservers( map @$_, @a );
-		my $reply = $res->send($query) || last;
-		$res->{callback}->($reply) if $res->{callback};
+		my @ns = $res->nameservers(@$_);
+
+		# uncoverable branch true	# depends on external data
+		next unless scalar @ns;
+
+		my $reply = $res->send($query);
+
+		# uncoverable branch true	# depends on external data
+		next unless $reply;
+
+		# uncoverable branch true	# depends on external data
+		next unless $reply->header->rcode eq 'NOERROR';
+
+		$self->_callback($reply);
 		return $reply;
 	}
 
 	foreach my $ns ( grep !ref($_), @$nslist ) {
-		print ";; find missing glue for $domain. ($ns)\n" if $res->{debug};
-		$res->empty_nameservers();
-		my @ip = $res->nameservers($ns);
-		$ns = [@ip];					# substitute IP list in situ
-		next unless @ip;
-		my $reply = $res->send($query) || next;
-		$res->{callback}->($reply) if $res->{callback};
+		$self->_diag(";; find missing glue for $ns\n");
+		$ns = [$res->nameservers($ns)];			# substitute IP list in situ
+
+		# uncoverable branch true	# depends on external data
+		next unless scalar @$ns;
+
+		my $reply = $res->send($query);
+
+		# uncoverable branch true	# depends on external data
+		next unless $reply;
+
+		# uncoverable branch true	# depends on external data
+		next unless $reply->header->rcode eq 'NOERROR';
+
+		$self->_callback($reply);
 		return $reply;
 	}
-	return;
 }
 
-sub query_dorecursion { &send; }	## historical
+
+sub query_dorecursion { &send; }				# uncoverable pod
 
 
 =head2 callback
@@ -190,8 +206,6 @@ For example to emulate dig's C<+trace> function:
     my $coderef = sub {
 	my $packet = shift;
 
-	$_->print for $packet->additional;
-
 	printf ";; Received %d bytes from %s\n\n",
 		$packet->answersize, $packet->answerfrom;
     };
@@ -204,24 +218,27 @@ for queries for missing glue records.
 =cut
 
 sub callback {
-	my ( $self, $sub ) = @_;
-
-	$self->{callback} = $sub if $sub && UNIVERSAL::isa( $sub, 'CODE' );
-	return $self->{callback};
-}
-
-sub recursion_callback { &callback; }	## historical
-
-
-sub bgsend {
 	my $self = shift;
-	my $class = ref($self) || $self;
-	Carp::croak "method ${class}::bgsend undefined";
+
+	( $self->{callback} ) = grep ref($_) eq 'CODE', @_;
 }
 
+sub _callback {
+	my $callback = shift->{callback};
+	$callback->(@_) if $callback;
+}
+
+sub recursion_callback { &callback; }				# uncoverable pod
+
 
 ########################################
 
+sub _diag {				## debug output
+	my $self = shift;
+	print @_ if $self->{debug};				# uncoverable branch true
+}
+
+
 sub _hints {				## default hints
 	require Net::DNS::ZoneFile;
 
@@ -232,8 +249,12 @@ sub _hints {				## default hints
 	my %auth = map { lc $_->nsdname => 1 } @auth;
 	my @glue = grep $auth{lc $_->name}, @rr;
 	my %glue;
-	foreach ( grep $_->type eq 'A',	   @glue ) { push @{$glue{lc $_->name}}, $_->address }
-	foreach ( grep $_->type eq 'AAAA', @glue ) { push @{$glue{lc $_->name}}, $_->address }
+	foreach ( grep $_->type eq 'A', @glue ) {
+		push @{$glue{lc $_->name}}, $_->address;
+	}
+	foreach ( grep $_->type eq 'AAAA', @glue ) {
+		push @{$glue{lc $_->name}}, $_->address;
+	}
 	my @ip = map @$_, values %glue;
 }
 
@@ -244,21 +265,36 @@ sub _hints {				## default hints
 =head1 ACKNOWLEDGEMENT
 
 This package is an improved and compatible reimplementation of the
-Net::DNS::Resolver::Recurse.pm created by Rob Brown in 2002.
-
-The contribution of Rob Brown is gratefully acknowledged.
+Net::DNS::Resolver::Recurse.pm created by Rob Brown in 2002,
+whose contribution is gratefully acknowledged.
 
 
 =head1 COPYRIGHT
 
-Copyright (c)2014 Dick Franks 
+Copyright (c)2014 Dick Franks.
 
-Portions Copyright (c)2002 Rob Brown 
+Portions Copyright (c)2002 Rob Brown.
 
 All rights reserved.
 
-This program is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
+
+=head1 LICENSE
+
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted, provided
+that the above copyright notice appear in all copies and that both that
+copyright notice and this permission notice appear in supporting
+documentation, and that the name of the author not be used in advertising
+or publicity pertaining to distribution of the software without specific
+prior written permission.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+DEALINGS IN THE SOFTWARE.
 
 
 =head1 SEE ALSO
